#!/usr/bin/python
import os, sys
from classGraph import Graph, Data, Clause
from toolLog import Log
import mappingTools as mappingTools
import pdb

org_parameters = None
if len(sys.argv) > 1:
    if os.path.isfile(sys.argv[1]):
        fp = open(sys.argv[1])
        org_parameters = mappingTools.readParameters(fp)
        fp.close()
    elif len(sys.argv) > 2:
        org_parameters = mappingTools.readParameters()
        org_parameters['id_exp'] = sys.argv[1]
        org_parameters['graph_file'] = sys.argv[2]
        if len(sys.argv) > 3:
            org_parameters['preds_file'] = sys.argv[3]
if org_parameters is None:
    Warning("No parameters given")
    exit()

data_parameters = dict(org_parameters)
mappingTools.openLog(data_parameters)
Graph.log = data_parameters['log']

expand_infos = []
collect_clauses = []
supports_dict = {}
data = Data()

#### Load predicates info from file 
if data_parameters.has_key('preds_file'):
    fp = open(data_parameters['data_rep'] + data_parameters['preds_file'])
    data.parsePreds(fp)
    fp.close()

#### Load data network from file 
if data_parameters.has_key('graph_file'):
    data.loadGraph(data_parameters['data_rep'] + data_parameters['graph_file'])
else:
    Warning("No data given")
    exit()

# Data.relevant = set(data.graph.elems_pred[Graph.ND]['relevant']['1'])

# data.writeGraph(data_parameters['data_rep'] + data_parameters['graph_file']+".db")
# exit()

#### Load init clauses from file 
if data_parameters.has_key('clauses_file'):
    clausesData = Data(data.toClausePreds())
    graphs, labels = clausesData.readGraphs(data_parameters['data_rep'] + data_parameters['clauses_file'])
    for key, graph in graphs.items():
        if key is not None:
            clause = Clause(key, graph, data.graph, data_parameters['query_nodes'])
            expand_infos.append({'other_side': None, 'turn':0, 'clause': len(collect_clauses)})
            supports_dict.setdefault(clause.supportKeyStr(data.graph), []).append(len(collect_clauses))
            collect_clauses.append(clause)

# #### Generate init clauses from predicates
includeP = True
if len(expand_infos) == 0:
    for pred, valsd in data.graph.elems_pred[Graph.EG].items():
        if includeP:
            for val in valsd.keys():
                clauses = mappingTools.initClauses(pred, val, data, data_parameters)
                for clause in clauses:
                    expand_infos.append({'other_side': None, 'turn':0, 'clause': len(collect_clauses)})
                    supports_dict.setdefault(clause.supportKeyStr(data.graph), []).append(len(collect_clauses))
                    collect_clauses.append(clause)

        # if pred == "publication":
        #     includeP = True

# #### Clauses for debuging
# # for (pred, val) in []:
# for (pred, val) in [('kinlink', "12")]:
# # for (pred, val) in [('Interacts_with', "EG")]:
#     clauses = mappingTools.initClauses(pred, val, data, data_parameters)
#     for clause in clauses:
#         expand_infos.append({'other_side': None, 'turn':0, 'clause': len(collect_clauses)})
#         supports_dict.setdefault(clause.supportKeyStr(data.graph), []).append(len(collect_clauses))
#         collect_clauses.append(clause)

if not os.path.exists(data_parameters['results_rep']):
    os.makedirs(data_parameters['results_rep'])

data_parameters["log"].printL(3,"##############SETTINGS")
data_parameters["log"].printL(3,"######################")
for (k,v) in data_parameters.items():
    data_parameters["log"].printL(3, "%s:%s" % (k, v))
data_parameters["log"].printL(3,"######################")


fh = open(data_parameters['results_rep']+("%s_clauses.bmg" % data_parameters['id_exp']), "w")
if data_parameters['print_supp']:
    fs = open(data_parameters['results_rep']+("%s_supports.txt" % data_parameters['id_exp']), "w")
else:
    fs = None
offset = 0

pi = 0
while pi < len(expand_infos):

    to_expand = expand_infos[pi]
    data_parameters["log"].printL(1,"######################")
    edges, parameters = mappingTools.prepareParameters(org_parameters, to_expand, collect_clauses, data, data_parameters['log'])

    if parameters is None:
        data_parameters["log"].printL(1,"Drop %s (%d, %d)..." % (to_expand['clause'], len(edges), to_expand['turn']))
    else:
        data_parameters["log"].printL(1,"Let's see %s (%d, %d)..." % (parameters['id_exp'], len(edges), to_expand['turn']))

        essential, details = mappingTools.searchClause(data.graph, edges, parameters)
        data_parameters["log"].printL(1,"Got %d out of %s..." % (len(essential), parameters['id_exp']))
        
        offset = mappingTools.writeShortResults(edges, details, parameters, fh, fs, offset)

        if data_parameters['redescribe']:
            for sid in essential:
                ### CHECK WE ARE NOT LOOPING AROUND (SUPPORT DIFFERS FROM ONE ITERATION BACK)
                clause = details['clauses'][sid]
                supp_key = clause.supportKeyStr(data.graph)
                inclus = True
                if supports_dict.has_key(supp_key):
                    inclus = False
                    # for pid in supports_dict[supp_key]:
                    #     if len(set(collect_clauses[pid].getTypesPred()) & set(clause.getTypesPred())) == 0:
                    #         inclus = True
                else:
                    supports_dict[supp_key] = []

                if inclus:
                    expand_infos.append({'other_side': to_expand['clause'], 'turn': to_expand['turn']+1, 'clause': len(collect_clauses)})
                else:
                    data_parameters["log"].printL(1,"### SEEMS WE ARE LOOPING ROUND")
                supports_dict[supp_key].append(len(collect_clauses))
                collect_clauses.append(clause)
    
    pi += 1

if data_parameters['redescribe']:
    fh.close()
